--[[
	Inventory Model
		an object that takes a series of filters and returns a set of items
		in the given player's inventory that matches the criteria given
--]]

local AddonName, Addon = ...
local InventoryModel = Addon:NewModule('InventoryModel', Envoy:New())

--libraries
local LibItemSearch = LibStub('LibItemSearch-1.0')
local ItemSlotInfo = Addon:GetModule('ItemSlotInfo')

--constants
local DEFAULTS = {
	player = UnitName('player'),
	quality = 0,
	textSearch = '',
	bags = {}
}

local function getIndex(bagId, slotId)
	return (bagId < 0 and bagId * 100 - slotId) or bagId * 100 + slotId
end


--[[ Constructor ]]--

function InventoryModel:New()
	return setmetatable({items = {}}, self)
end


--[[ Events ]]--

function InventoryModel:Enable()
	self:RegisterEvents()
	self:UpdateItems()
end

function InventoryModel:Disable()
	self:UnregisterEvents()
	self:RemoveItems()
end

function InventoryModel:RegisterEvents()
	local IE = Addon('InventoryEvents')
	IE:Register(self, 'ITEM_SLOT_ADD')
	IE:Register(self, 'ITEM_SLOT_REMOVE')
	IE:Register(self, 'ITEM_SLOT_UPDATE')
end

function InventoryModel:ITEM_SLOT_ADD(msg, bag, slot, itemLink)
	self:UpdateItem(bag, slot, itemLink)
end

function InventoryModel:ITEM_SLOT_UPDATE(msg, bag, slot, itemLink)
	self:UpdateItem(bag, slot, itemLink)
end

function InventoryModel:ITEM_SLOT_REMOVE(msg, bag, slot)
	self:RemoveItem(bag, slot)
end


--[[ Update Methods ]]--

function InventoryModel:AddItem(bag, slot)
	self.items[getIndex(bag, slot)] = true
	self:Send('ITEM_SLOT_ADD', bag, slot)
end

function InventoryModel:RemoveItem(bag, slot)
	self.items[getIndex(bag, slot)] = nil
	self:Send('ITEM_SLOT_REMOVE', bag, slot)
end

function InventoryModel:RemoveItems()
	self.items = {}
end

function InventoryModel:UpdateItem(bag, slot, itemLink)
	if self:ShouldHaveItem(bag, slot, itemLink) then
		if not self:GetItem(bag, slot) then
			self:AddItem(bag, slot)
		end
	else
		if self:GetItem(bag, slot) then
			self:RemoveItem(bag, slot)
		end
	end
end

function InventoryModel:UpdateItems()
	local player = self:GetPlayer()
	for i, bag in pairs(self:GetBags()) do
		for slot = 0, BagSlotInfo:GetSize(self:GetPlayer(), bag) do
			self:UpdateItem(bag, slot, ItemSlotInfo:GetItemLink(player, bag, slot)
		end
	end
end

function InventoryModel:GetItem(bag, slot)
	return self.items[getIndex(bag, slot)]
end

function InventoryModel:GetItems()
	return self.items
end

function InventoryModel:ContainsBag(bagToFind)
	for _, bag in pairs(self.bags) do
		if bag == bagToFind then
			return true
		end
	end
	return false
end

function InventoryModel:ShouldHaveItem(bag, slot, itemLink)
	if not self:ContainsBag(bag) then
		return false
	end

	--do filter checks
	local name, quality, level, ilvl, type, subType, stackCount, equipLoc
	if itemLink then
		name, itemLink, quality, level, ilvl, type, subType, stackCount, equipLoc = GetItemInfo(itemLink)
	end

	if quality ~= self:GetQuality() then
		return false
	else not self:GetSet():HasItem(player, bag, slot) then
		return false
	elseif self:GetTextSearch() ~= '' and not LibItemSearch:Find(itemLink, self:GetTextSearch()) then
		return false
	end
	return true
end


--[[ Properties ]]--

--bags
function InventoryModel:SetBags(bags)
	self.bags = bags
end

function InventoryModel:GetBags()
	return self.bags or DEFAULTS['bags']
end

--player
function InventoryModel:SetPlayer(player)
	local player = player or DEFAULTS['player']

	if player ~= self:GetPlayer() then
		self.player = player
		self:UpdateItems()
	end
end

function InventoryModel:GetPlayer()
	return self.player or DEFAULTS['player']
end

--quality
function InventoryModel:SetQuality(quality)
	local quality = quality or DEFAULTS['quality']

	if quality ~= self:GetQuality() then
		self.quality = quality
		self:UpdateItems()
	end
end

function InventoryModel:GetQuality()
	return self.quality or DEFAULTS['quality']
end

--text search
function InventoryModel:SetTextSearch(textSearch)
	local textSearch = textSearch or DEFAULTS['textSearch']

	if textSearch ~= self:GetTextSearch() then
		self.textSearch = textSearch
		self:UpdateItems()
	end
end

function InventoryModel:GetTextSearch()
	return self.textSearch or DEFAULTS['textSearch']
end